@@ -1,12 +1,13 @@
 import abc
 
 from cvargparse import Arg
+from cvargparse import BaseParser
 from cvfinetune.parser.utils import DEFAULT_INFO_FILE
 from cvfinetune.parser.utils import get_info_file
 from cvfinetune.parser.utils import parser_extender
 
 @parser_extender
-def add_dataset_args(parser):
+def add_dataset_args(parser: BaseParser) -> None:
 
 	info_file = get_info_file()
 
