@@ -1,27 +1,27 @@
 import abc
 
-from cvargparse import Arg
-from cvfinetune.parser.utils import get_info_file
-from cvfinetune.parser.utils import parser_extender
-
 from chainer_addons.links import PoolingType
 from chainer_addons.models import PrepareType
+from cvargparse import Arg
+from cvargparse import BaseParser
+from cvfinetune.parser.utils import parser_extender
+from cvmodelz.models import ModelFactory
 
 @parser_extender
-def add_model_args(parser):
-
-	info_file = get_info_file()
-
-	if info_file is None:
-		model_type_choices = None
-	else:
-		model_type_choices = info_file.MODELS.keys()
+def add_model_args(parser: BaseParser) -> None:
 
+	choices = ModelFactory.get_models(["chainercv2", "cvmodelz"])
 	_args = [
 		Arg("--model_type", "-mt",
-			default="resnet", choices=model_type_choices,
+			required=True,
+			choices=choices,
 			help="type of the model"),
 
+		Arg("--pre_training", "-pt",
+			default="imagenet",
+			choices=["imagenet", "inat"],
+			help="type of model pre-training"),
+
 		Arg("--input_size", type=int, nargs="+", default=0,
 			help="overrides default input size of the model, if greater than 0"),
 
@@ -45,6 +45,9 @@ def add_model_args(parser):
 
 		Arg("--load_strict", action="store_true",
 			help="load weights in a strict mode"),
+
+		Arg("--load_path", type=str, default="",
+			help="load path within the weights archive"),
 	]
 
 	parser.add_args(_args, group_name="Model arguments")
