@@ -2,12 +2,13 @@ import abc
 
 from cvargparse import Arg
 from cvargparse import ArgFactory
+from cvargparse import BaseParser
 from cvfinetune.parser.utils import parser_extender
 
 from chainer_addons.training import OptimizerType
 
 @parser_extender
-def add_training_args(parser):
+def add_training_args(parser: BaseParser) -> None:
 
 	_args = ArgFactory([
 
