@@ -1,6 +1,7 @@
 import logging
 from os.path import join, basename
 from datetime import datetime
+from typing import Tuple
 
 import chainer
 from chainer.training import extensions, Trainer as T
@@ -28,70 +29,40 @@ def _is_adam(opts):
 	return opts.optimizer == OptimizerType.ADAM.name.lower()
 
 class Trainer(T):
-	_default_base_model = "model"
 
-	def __init__(self, opts, updater, evaluator=None, weights=None, intervals=default_intervals, no_observe=False):
-
-		self._only_eval = opts.only_eval
-		if weights is None or weights == "auto":
-			self.base_model = self._default_base_model
-		else:
-			self.base_model, _, _ = basename(weights).rpartition(".")
-
-		optimizer = updater.get_optimizer("main")
-		# adam has some specific attributes, so we need to check this
-		is_adam = _is_adam(opts)
-		clf = optimizer.target
-		model = clf.model
-
-		if no_observe:
-			outdir = opts.output
-		else:
-			outdir = self.output_directory(opts)
-			logging.info("Training outputs are saved under \"{}\"".format(outdir))
+	def __init__(self, opts, updater,
+		evaluator: extensions.Evaluator = None,
+		intervals: attr_dict = default_intervals,
+		no_observe: bool = False):
 
 		super(Trainer, self).__init__(
 			updater=updater,
 			stop_trigger=(opts.epochs, 'epoch'),
-			out=outdir
+			out=opts.output
+		)
+		logging.info("Training outputs are saved under \"{}\"".format(self.out))
+
+		self._only_eval = opts.only_eval
+		self.offset = 0
+
+		self.setup_evaluator(evaluator, intervals.eval)
+
+		self.setup_warm_up(epochs=opts.warm_up,
+			after_warm_up_lr=opts.learning_rate,
+			warm_up_lr=opts.learning_rate
 		)
 
-		### Evaluator ###
-		self.evaluator = evaluator
-		if evaluator is not None:
-			self.extend(evaluator, trigger=intervals.eval)
+		self.setup_lr_schedule(
+			lr=opts.learning_rate,
+			lr_target=opts.lr_target,
+			lr_shift_trigger=(opts.lr_shift, "epoch"),
+			lr_decrease_rate=opts.lr_decrease_rate,
 
-		### Warm up ###
-		lr_offset = 0
-		if opts.warm_up:
-			assert opts.warm_up > 0, "Warm-up argument must be positive!"
-			lr_offset = opts.warm_up
-
-			warm_up_lr = opts.learning_rate
-			logging.info("Warm-up of {} epochs enabled!".format(opts.warm_up))
-			self.extend(WarmUp(
-				opts.warm_up, model,
-				opts.learning_rate, warm_up_lr))
-
-
-		### LR shift ###
-		if opts.cosine_schedule is not None and opts.cosine_schedule > 0:
-			lr_shift_ext = CosineAnnealingLearningRate(
-				attr="alpha" if is_adam else "lr",
-				lr=opts.learning_rate,
-				target=opts.lr_target,
-				epochs=opts.epochs,
-				offset=lr_offset,
-				stages=opts.cosine_schedule
-			)
-			new_epochs = lr_shift_ext._epochs
-			self.stop_trigger = trigger_module.get_trigger((new_epochs, "epoch"))
-			self.extend(lr_shift_ext)
-		else:
-			lr_shift_ext = lr_shift(optimizer,
-				init=opts.learning_rate,
-				rate=opts.lr_decrease_rate, target=opts.lr_target)
-			self.extend(lr_shift_ext, trigger=(opts.lr_shift, 'epoch'))
+			# needed for cosine annealing
+			epochs=opts.epochs,
+			cosine_schedule=opts.cosine_schedule,
+			attr="alpha" if _is_adam(opts) else "lr",
+		)
 
 		### Code below is only for "main" Trainers ###
 		if no_observe: return
@@ -100,11 +71,12 @@ class Trainer(T):
 		self.extend(extensions.LogReport(trigger=intervals.log))
 
 		### Snapshotting ###
-		self.setup_snapshots(opts, clf.model, intervals.snapshot)
+		self.setup_snapshots(opts, self.model, intervals.snapshot)
 
 		### Reports and Plots ###
 		print_values, plot_values = self.reportables(opts)
-		self.extend(extensions.PrintReport(print_values), trigger=intervals.print)
+		self.extend(extensions.PrintReport(print_values),
+			trigger=intervals.print)
 		for name, values in plot_values.items():
 			ext = extensions.PlotReport(values, 'epoch', file_name='{}.png'.format(name))
 			self.extend(ext)
@@ -113,6 +85,77 @@ class Trainer(T):
 		if not opts.no_progress:
 			self.extend(extensions.ProgressBar(update_interval=1))
 
+		elif self.evaluator is not None:
+			self.evaluator._progress_bar = False
+
+	@property
+	def optimizer(self):
+		return self.updater.get_optimizer("main")
+
+	@property
+	def clf(self):
+		return self.optimizer.target
+
+	@property
+	def model(self):
+		return self.clf.model
+
+	def setup_lr_schedule(self,
+		lr: float,
+		lr_target: float,
+		lr_shift_trigger: Tuple[int, str],
+		lr_decrease_rate: float,
+
+		epochs: int,
+		cosine_schedule: int,
+		attr: str):
+
+		if cosine_schedule is not None and cosine_schedule > 0:
+			lr_shift_ext = CosineAnnealingLearningRate(
+				attr=attr,
+				lr=lr,
+				target=lr_target,
+				epochs=epochs,
+				offset=self.offset,
+				stages=cosine_schedule
+			)
+			new_epochs = lr_shift_ext._epochs
+			self.stop_trigger = trigger_module.get_trigger((new_epochs, "epoch"))
+			logging.info(f"Changed number of training epochs from {epochs} to {new_epochs}")
+			lr_shift_trigger = None
+
+		else:
+			lr_shift_ext = lr_shift(self.optimizer,
+				init=lr, rate=lr_decrease_rate, target=lr_target)
+
+		self.extend(lr_shift_ext, trigger=lr_shift_trigger)
+
+	def setup_warm_up(self, epochs: int, after_warm_up_lr: float, warm_up_lr: float):
+
+		if epochs is None or epochs == 0:
+			return
+
+		assert epochs > 0, "Warm-up argument must be positive!"
+		self.offset = epochs
+
+		logging.info(f"Warm-up of {epochs} epochs enabled!")
+
+		self.extend(WarmUp(epochs, self.model,
+			initial_lr=after_warm_up_lr,
+			warm_up_lr=warm_up_lr
+			)
+		)
+
+
+	def setup_evaluator(self,
+		evaluator: extensions.Evaluator,
+		trigger: Tuple[int, str]):
+
+		self.evaluator = evaluator
+		if evaluator is None:
+			return
+		self.extend(evaluator, trigger=trigger)
+
 	def setup_snapshots(self, opts, obj, trigger):
 
 		if opts.no_snapshot:
@@ -152,16 +195,6 @@ class Trainer(T):
 		return print_values, plot_values
 
 
-	def output_directory(self, opts):
-
-		result = opts.output
-
-		if self.base_model != self._default_base_model:
-			result = join(result, self.base_model)
-
-		result = join(result, datetime.now().strftime("%Y-%m-%d-%H.%M.%S.%f"))
-		return result
-
 	def run(self, init_eval=True):
 		if init_eval:
 			logging.info("Evaluating initial model ...")
